////////////////////////////////////////////////////////////////////////////////
//    Scorched3D (c) 2000-2003
//
//    This file is part of Scorched3D.
//
//    Scorched3D is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Scorched3D is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Scorched3D; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////

#if !defined(__INCLUDE_CameraPositionActionh_INCLUDE__)
#define __INCLUDE_CameraPositionActionh_INCLUDE__

#include <engine/ActionMeta.h>
#include <common/Vector.h>
#include <set>

class CameraPositionAction : public ActionMeta
{
public:
	CameraPositionAction();
	CameraPositionAction(Vector &showPosition,
		float showTime,
		unsigned int priority);
	virtual ~CameraPositionAction();

	virtual void init();
	virtual void simulate(float frameTime, bool &remove);
	virtual bool writeAction(NetBuffer &buffer);
	virtual bool readAction(NetBufferReader &reader);

	REGISTER_ACTION_HEADER(CameraPositionAction);

	float getShowTime() { return showTime_; }
	float getStartTime() { return startTime_; }
	Vector &getShowPosition() { return showPosition_; }
	unsigned int &getShowPriority() { return showPriority_; }

protected:
	float totalTime_;

	float startTime_;
	float showTime_;
	unsigned int showPriority_;
	Vector showPosition_;
};

class CameraPositionActionRegistry
{
public:
	static void addCameraPositionAction(CameraPositionAction *action);
	static void rmCameraPositionAction(CameraPositionAction *action);

	static CameraPositionAction *getCurrentAction();

protected:
	static std::set<CameraPositionAction *> actions_;
	static CameraPositionAction *currentAction_;

	static CameraPositionAction *getCurrentBest();
};

#endif
